/* $Id: 16550.c,v 1.4 2000/11/15 17:44:54 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <console.h>
#include <malloc.h>
#include <board.h>

#define RBR 0
#define THR 0
#define IER 1
#define IIR 2
#define FCR 2
#define LCR 3
#define MCR 4
#define LSR 5
#define MSR 6
#define SCR 7
#define DLL 0
#define DLM 1

static void outchar_16550(void *data, char c);
static int inchar_16550(void *data);
RODATA static const gbios_console_fn fn_16550 = {
    outchar_16550, inchar_16550
}; 

struct data_16550 {
    volatile unsigned char *base;
    int spacing;
    int fifo_level;
    unsigned hertz;
};


static inline void write_16550(struct data_16550 *data, int reg,
            unsigned char x)
{
    data->base[reg*data->spacing] = x;
}
static inline unsigned char read_16550(struct data_16550 *data, int reg)
{
    return data->base[reg*data->spacing];
}

static void init_16550(struct data_16550 *data)
{
    unsigned divisor = data->hertz / (9600 * 16);
    write_16550(data, LCR, 0x83);
    /* 9600 baud */
    write_16550(data, DLM, divisor >> 8);
    write_16550(data, DLL, divisor & 0xff);
    /* Turn off divisor access */
    write_16550(data, LCR, 0x03);
    /* Set polled mode */
    write_16550(data, IER, 0);
    /* Enable and clear FIFOs */
    write_16550(data, FCR, 0x07);
    write_16550(data, MCR, 0x03);
}

int init_console_16550(unsigned base, unsigned hertz, int spacing)
{
    struct data_16550 *data = malloc(sizeof(struct data_16550));
    
    if (data)
    {
        data->base = (volatile unsigned char *)base;
        data->spacing = spacing;
        data->hertz = hertz;
        
        init_16550(data);
        register_console_device(&fn_16550, "16550", data);
        return 0;
    }
    else
        return -1;
}

void outchar_16550(void *data, char c)
{
     struct data_16550 *d16550 = data;

      while((read_16550(d16550, LSR) & 0x40) == 0)
             ;
      write_16550(d16550, THR, c);
}

int inchar_16550(void *data)
{
    int result = -1;
    struct data_16550 *d16550 = data;
    
    if (read_16550(d16550, LSR) & 0x01)
             result = read_16550(d16550, RBR);
    
    return result;
}
