/* $Id: ata.c,v 1.2 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <stdio.h>
#include <timer.h>
unsigned short ata_buffer[256];

typedef struct ata_disk
{
    volatile char *ata_reg;
    void (*read_data) (struct ata_disk *, void *);
    void (*read_id) (struct ata_disk *, void *);
    int lba;
    unsigned capacity;
    unsigned sectors;
    unsigned heads;
    unsigned char drive;
}
ata_disk;


void ata_identify_error (ata_disk * disk)
{
    printf ("Error 0x%x ", disk->ata_reg[1]);
    disk->ata_reg[7] = 3;
    while (disk->ata_reg[7] & 0x80)
	;
    printf ("Error 0x%x\n", disk->ata_reg[1]);
}

int ata_read_pio (ata_disk * disk, void *data)
{
    while ((disk->ata_reg[7] & 0x88) == 0x80)
	;

    disk->read_data (disk, data);

    if (disk->ata_reg[7] & 1)
    {
	ata_identify_error (disk);
	return -1;
    }
    return 0;
}

void tpe4_read_data (ata_disk * disk, void *data)
{
    char *target = data;
    int i;

    for (i = 0; i < 512; i++)
    {
	target[i] = disk->ata_reg[8];
    }
}

void tpe4_read_id (ata_disk * disk, void *data)
{
    char *target = data;
    int i;

    for (i = 0; i < 512; i += 2)
    {
	target[i + 1] = disk->ata_reg[8];
	target[i] = disk->ata_reg[9];
    }
}

int ata_read_wpio (ata_disk * disk, void *data)
{

    while ((disk->ata_reg[7] & 0x88) == 0x80)
	;

    disk->read_id (disk, data);
    if (disk->ata_reg[7] & 1)
    {
	ata_identify_error (disk);
	return -1;
    }
    return 0;
}

void ata_read_sector (ata_disk * disk, void *data, unsigned block,
		      unsigned count)
{
    unsigned j;
    unsigned t1,
     t2;
    unsigned char *target = data;

    t1 = ticks_since_boot ();

    if (disk->lba)
    {
	disk->ata_reg[6] = 0xe0 | disk->drive | (block >> 24);
	disk->ata_reg[5] = (block >> 16) & 0xff;
	disk->ata_reg[4] = (block >> 8) & 0xff;
	disk->ata_reg[3] = block & 0xff;
    }
    else
    {
	unsigned int sect,
	 head,
	 cyl,
	 track;

	track = block / disk->sectors;
	sect = block % disk->sectors + 1;
	head = track % disk->heads;
	cyl = track / disk->heads;
	disk->ata_reg[6] = head | disk->drive;
	disk->ata_reg[5] = cyl >> 8;
	disk->ata_reg[4] = cyl & 0xff;
	disk->ata_reg[3] = sect;
	printf ("Block %d = C %d H %d S %d\n", block, cyl, head, sect);
    }
    disk->ata_reg[2] = count;
    disk->ata_reg[7] = 0x21;
    for (j = 0; j < count; j++)
    {
	if (ata_read_pio (disk, target + 512 * j) < 0)
	    return;
    }
    t2 = ticks_since_boot ();
    /*  printf ("Ticks %d kByte/s %d\n", t2 - t1,
     * (int) (count * (double) get_hz () / (2.0 * (t2 - t1)))); */
}

#if 1
static unsigned char boot_sector[512];

void decode_fat12 (unsigned char *sector /* unsigned start, unsigned length */ )
{
    unsigned char name[0xc];
    int i;
    unsigned char *bpb = sector + 0xb;
    

    /*  ata_read_sector(boot_sector, start, 1); */
    memcpy (name, sector + 3, 8);
    name[8] = 0;
    printf ("FAT12 partition at (%s)\n", name);
    printf ("%d bytes per sector\n", bpb[0] | (bpb[1] << 8));
    printf ("%d sectors per cluster\n", bpb[2]);
    printf ("signature 0x%02x%02x\n", sector[0x1fe], sector[0x1ff]);
    printf ("Reserved sectors %d\n", bpb[3]);
    printf("FATs: %d\n", bpb[5]);
    printf("Sectors per FAT: %d\n", bpb[0xb]);
    if (bpb[8] | bpb[9])
        printf("Total sectors %d\n", (unsigned)bpb[8] | (bpb[9] << 8));
    else
        printf("Total sectors %d\n",
                (unsigned)bpb[0x15] | (bpb[0x16] << 8) | (bpb[0x17] << 16) |
                bpb[0x18] << 24);
    memcpy(name, bpb + 0x20, 0xb);
    name[0xb] = 0;
    printf("Label (%s)\n", name);
    
    for(i = 0; i < 32; i++)
    {
        printf("%02x ", sector[i]);
        
        if ((i & 31) == 31)
            printf("\n");
        
    }
}

unsigned chs_to_block(ata_disk *disk, const unsigned char *chs)
{
    unsigned c, h, s;
    unsigned block;
    h = chs[0];
    s = chs[1] & 0x3f;
    c = chs[2] | ((chs[1] & 0xc0) << 2);
    block = c*disk->heads*disk->sectors + h*disk->sectors + s - 1;
    printf("c %d h %d s %d = block %d\n", c, h, s, block);
    return block;
}

void decode_partition (ata_disk * disk, const char *partition)
{
    unsigned start,
     length;

    if (disk->lba)
    {
	start = partition[8] |
	    (partition[9] << 8) | (partition[10] << 16) | (partition[11] <<
							   24);
	length =
	    partition[12] | (partition[13] << 8) | (partition[14] << 16) |
	    (partition[15] << 24);
    }
    else
    {
        start = chs_to_block(disk, partition + 1);
        length = chs_to_block(disk, partition + 5) - start;
    }
    
    	printf ("Partition type 0x%x start %d length %d (%d kByte)\n",
		partition[4], start, length, length / 2);
	if (partition[4] == 1)
	{
	    ata_read_sector (disk, boot_sector, start, 1);
	    decode_fat12 (boot_sector);
	}
    
}

void read_partition (ata_disk * disk)
{
    char *c_buffer = (char *) ata_buffer;

    ata_read_sector (disk, ata_buffer, 0, 1);
    printf ("Partition signature 0x%x\n", ata_buffer[255]);
    if (ata_buffer[255] == 0x55aa)
    {
	printf ("c_buffer_sig == 0x%x 0x%x\n", c_buffer[510],
		c_buffer[511]);
	decode_partition (disk, c_buffer + 446);
	decode_partition (disk, c_buffer + 462);
	decode_partition (disk, c_buffer + 478);
	decode_partition (disk, c_buffer + 494);

    }
}
#endif

extern char *TftpLoadAddress;

void ata_identify (ata_disk * disk)
{
    char string[41];

    disk->ata_reg[6] = disk->drive;
    disk->ata_reg[7] = 0xec;

    ata_read_wpio (disk, ata_buffer);

    memcpy (string, ata_buffer + 27, 40);
    string[40] = 0;
    printf ("Serial: (%s)\n", string);

    memcpy (string, ata_buffer + 10, 20);
    string[20] = 0;
    printf ("Serial: (%s)\n", string);

    printf ("Modes 0x%04x\n", ata_buffer[49]);
    disk->lba = ata_buffer[49] & 0x200;
    if (disk->lba)
    {
	disk->ata_reg[6] = 0x40;
	if (!(disk->ata_reg[6] & 0x40))
	{
	    printf ("LBA mode broken\n");
	    disk->lba = 0;
	}
    }
    disk->lba = 0;
    if (disk->lba)
    {
	disk->capacity = ata_buffer[60] | (ata_buffer[61] << 16);
	printf ("LBA sectors = %d, total size = %d bytes\n",
		disk->capacity, 512 * disk->capacity);
    }
    else if (ata_buffer[53] & 1)
    {
	disk->capacity = ata_buffer[57] | (ata_buffer[58] << 16);
	printf ("Cylinders %d Heads %d Sectors %d, Capacity %d\n",
		ata_buffer[54], ata_buffer[55], ata_buffer[56],
		512 * disk->capacity);
	disk->sectors = ata_buffer[56];
	disk->heads = ata_buffer[55];
    }
}



void ata_test (void)
{
    ata_disk compact_flash;

    compact_flash.ata_reg = (void *) 0xff600000;
    compact_flash.read_data = tpe4_read_data;
    compact_flash.read_id = tpe4_read_id;
    compact_flash.drive = 0;

    ata_identify (&compact_flash);
    ata_read_sector (&compact_flash, 0, 20, 255);
    read_partition (&compact_flash);
    /*  for(i = 20; i < 40; i++)
     * {
     * printf("%d ", i);
     * ata_read_sector(TftpLoadAddress,i, 1);
     * }
     * 
     * printf("65536 ");
     * ata_read_sector(TftpLoadAddress,65536, 1); 
     read_partition (); */

}
