/* $Id: amd800.c,v 1.3 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <flash.h>
#include <stdio.h>
#include <malloc.h>

struct data_amd800
{
    unsigned base;
    unsigned spacing;
    unsigned cs;
    const char *tag;
};

static const char *identify_amd (void *data);
static int erase_flash_amd800 (void *data, unsigned offset,

			       unsigned length);
static int program_flash_amd800 (void *data, unsigned offset, void *source,

				 unsigned length);

static gbios_flash_fn fn_amd800 = {
    identify_amd,
    NULL,
    NULL,
    erase_flash_amd800,
    program_flash_amd800
};

const char *identify_amd (void *data)
{
    struct data_amd800 *d800 = data;

    if (!d800->tag)
    {
	volatile unsigned char *flash =

	    (volatile unsigned char *) d800->base;
	unsigned char type,
	 id;

	*(flash + 0xaaa * d800->spacing) = 0xaa;
	*(flash + 0x555 * d800->spacing) = 0x55;
	*(flash + 0xaaa * d800->spacing) = 0x90;
	type = *(flash + 2 * d800->spacing);
	id = *flash;
	*flash = 0xf0;
	if (id == 1 && type == 0x5b)
	{
	    d800->cs = 45;
	    d800->tag = "Am29LV800BB";
	}
	else
	{
	    printf ("Unknown flash ID: 0x%02x 0x%02x\n", id, type);
	}
    }
    return d800->tag;
}

int erase_flash_amd800 (void *data, unsigned offset, unsigned length)
{
    struct data_amd800 *d800 = data;
    volatile unsigned char *flash = (volatile unsigned char *) d800->base;
    volatile unsigned char *flash_aaa = flash + 0xaaa * d800->spacing;
    volatile unsigned char *flash_555 = flash + 0x555 * d800->spacing;
    int id;
    int cs = 9999;

    printf ("Erase from 0x%08x to 0x%08x\n", offset, offset + length);
    *flash_aaa = 0xAA;		// Chip Erase
    *flash_555 = 0x55;
    *flash_aaa = 0x80;
    *flash_aaa = 0xAA;
    *flash_555 = 0x55;
    *flash_aaa = 0x10;

    for (; cs > 0; cs--)
    {
	id = *(flash + 16);
	if (id & 0xA0)		// DQ7 or DQ5 set: done or error
	    break;
	printf ("%4d\b\b\b\b", cs);
    }

    *flash_aaa = 0xF0;		// In case of error

    printf ("\b\b\b\b    \b\b\b\b");
    if (cs == 0)
    {
	printf ("Could not erase flash, timeout.\n");
	return -1;
    }
    else if ((id & 0x80) == 0)
    {
	printf ("Could not erase flash, status=%02x.\n", id);
	return -1;
    }
    printf ("Flash erased\n");
    return 0;
}

int init_flash_amd800 (char *tag, unsigned base, unsigned spacing)
{
    struct data_amd800 *data = malloc (sizeof (struct data_amd800));

    if (data)
    {
	data->base = base;
	data->spacing = spacing;
	data->tag = NULL;
	if (identify_amd (data))
	    register_flash_device (&fn_amd800, tag, data, base,
				   1024 * 1024);
	else
	{
	    free (data);
	    return -1;
	}
    }
    else
	return -1;
    return 0;
}

int program_flash_amd800 (void *data, unsigned offset, void *source,
			  unsigned length)
{
    struct data_amd800 *d800 = data;
    volatile unsigned char *flash = (volatile unsigned char *) d800->base;
    volatile unsigned char *flash_aaa = flash + 0xaaa * d800->spacing;
    volatile unsigned char *flash_555 = flash + 0x555 * d800->spacing;
    int id;
    int cs;
    int errs = 0;
    volatile char *s;
    volatile char *d;

    printf ("Program from 0x%08x to 0x%08x\n", offset, offset + length);
    printf ("Data at %p\n", source);

    *flash_aaa = 0xAA;		// Unlock Bypass
    *flash_555 = 0x55;
    *flash_aaa = 0x20;

    s = (unsigned char *) source;
    d = flash + offset * d800->spacing;
    cs = length;

    while (cs > 0 && !errs)
    {
	*flash = 0xA0;	// Unlock Bypass Program
	*d = *s;		// Program data

	while (1)
	{
	    id = *d;
	    if ((id & 0x80) == (*s & 0x80))	// DQ7 right? => program done
		break;
	    else if (id & 0x20)
	    {			// DQ5 set? => errors
		errs++;
		break;
	    }
	}

	// PRINT("Set %08lx = %02x\n", d, *d);

	s += 1;
	d += d800->spacing;
	cs--;
    }

    *flash = 0x90;		// Unlock Bypass Program Reset
    *flash = 0x00;
    *flash = 0xF0;

    if (errs != 0)
    {
	printf ("FAIL: Status=%02x.\n", id);
	return -1;
    }
    printf ("OK.\n");


    // Step 4: Verify the flash.

    printf ("  Verifying flash : ...");
    errs = 0;
    s = (unsigned char *) source;
    d = flash + offset * d800->spacing;
    for (cs = 0; cs < length; cs++)
    {
	if (*s != *d)
	{
	    if (errs == 0)
		printf ("ERROR: Addr: %08lx, PCI: %02x Lcl: %02x.\n",
		       s, *s, *d);
	    errs++;
	}
	s += 1;
	d += d800->spacing;
    }

    if (errs == 0)
	printf ("OK.\n");
    else
    {
	printf ("  FAIL: %d errors.\n", errs);
	return -1;
    }

    return 0;
}
