/* $Id: atmel49040.c,v 1.4 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <flash.h>
#include <stdio.h>
#include <malloc.h>
#include <timer.h>
#include <console.h>

struct data_atmel
{
    unsigned base;
    unsigned spacing;
    const char *tag;
};

static const char *identify_atmel (void *data);
static int erase_flash_atmel (void *data, unsigned offset,
			       unsigned length);
static int program_flash_atmel (void *data, unsigned offset, void *source,
				 unsigned length);

static gbios_flash_fn fn_atmel = {
    identify_atmel,
    NULL,
    NULL,
    erase_flash_atmel,
    program_flash_atmel
};

const char *identify_atmel (void *data)
{
    struct data_atmel *d = data;

    if (!d->tag)
    {
	volatile unsigned char *flash =
	    (volatile unsigned char *) d->base;
	unsigned char type,
	 id;

	*(flash + 0x5555 * d->spacing) = 0xaa;
	*(flash + 0x2aaa * d->spacing) = 0x55;
	*(flash + 0x5555 * d->spacing) = 0x90;
	type = *(flash + d->spacing);
	id = *flash;
	*flash = 0xf0;
	if (id == 0x1f && type == 0x13)
	{
	    d->tag = "Atmel49LV040";
	}
	else
	{
	    printf ("Unknown flash ID: 0x%02x 0x%02x\n", id, type);
	}
    }
    return d->tag;
}

int init_flash_atmel040 (char *tag, unsigned base, unsigned spacing)
{
    struct data_atmel *data = malloc (sizeof (struct data_atmel));

    if (data)
    {
	data->base = base;
	data->spacing = spacing;
	data->tag = NULL;
	if (identify_atmel (data))
	    register_flash_device (&fn_atmel, tag, data, base,
				   1024 * 1024);
	else
	{
	    free (data);
	    return -1;
	}
    }
    else
	return -1;
    return 0;
}

int erase_flash_atmel (void *data, unsigned offset, unsigned length)
{
    struct data_atmel *d = data;

    if (d)
    {
        unsigned t1, prev;
        int spin = 0;
	volatile unsigned char *flash =
	    (volatile unsigned char *) d->base;
	*(flash + 0x5555 * d->spacing) = 0xaa;
	*(flash + 0x2aaa * d->spacing) = 0x55;
	*(flash + 0x5555 * d->spacing) = 0x80;
	*(flash + 0x5555 * d->spacing) = 0xaa;
	*(flash + 0x2aaa * d->spacing) = 0x55;
	*(flash + 0x5555 * d->spacing) = 0x10;
        
        t1 = ticks_since_boot();
        prev = 0;
        while((*flash != 0xff) && (ticks_since_boot() < t1 + 15*get_hz()))
        {
            if (ticks_since_boot() - prev > get_hz() / 10)
            {
                print_spinner(spin++);
                prev = ticks_since_boot();
            }
        }
        if (*flash != 0xff)
            return -1;
    }
    return 0;
}

static int program_byte(struct data_atmel *d, 
        unsigned offset, unsigned char byte)
{
    unsigned t1;
	volatile unsigned char *flash =
	    (volatile unsigned char *) d->base;
	*(flash + 0x5555 * d->spacing) = 0xaa;
	*(flash + 0x2aaa * d->spacing) = 0x55;
	*(flash + 0x5555 * d->spacing) = 0xa0;
        *(flash + offset * d->spacing) = byte;
        t1 = ticks_since_boot();
        while((*(flash + offset * d->spacing) != byte) &&
                ticks_since_boot() < t1 + get_hz())
            ;
        if (*(flash + offset * d->spacing) != byte)
        {
            printf("failed at offset %d, got 0x%02x expected 0x%02x\n",
                    offset, *(flash + offset * d->spacing), byte);
            return -1;
        }
        return 0;
}

int program_flash_atmel (void *data, unsigned offset, void *source,
			  unsigned length)
{
    unsigned char *s = source;
    int spin = 0;
    while(length)
    {
        if (program_byte(data, offset, *s) < 0)
            return -1;
        s++;
        length--;
        offset++;
        if ((offset & 0x1f) == 1)
            print_spinner(offset / 32);
    }
    return 0;
}
