/* $Id: flash.c,v 1.5 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <flash.h>
#include <malloc.h>
#include <dictionary.h>
#include <stdio.h>
#include <iotypes.h>
#include <net.h>

static gbios_flash_device *first_flash;
static void list_flash_devices(int argc, char **argv, void *data);
static void progself_flash_devices(int argc, char **argv, void *data);
static void elf_flash_device(int argc, char **argv, void *data);

gbios_dictionary flash_dictionary[] = {
    {"list", "List FLASH devices", list_flash_devices, NULL},
    {"elf", "Program loaded GBIOS into device", elf_flash_device, NULL},
    {"self", "Program current GBIOS into device", progself_flash_devices, NULL},
    {NULL}
};

gbios_helpdata flash_helpdata = {
    "FLASH Commands\n", flash_dictionary, NULL
};

int register_flash_device (gbios_flash_fn * fn, char *tag, void *data,
        unsigned base, unsigned size)
{
    gbios_flash_device *device = malloc (sizeof (gbios_flash_device));

    if (device)
    {
	device->next = first_flash;
	first_flash = device;
	device->fn = fn;
	device->tag = tag;
	device->data = data;
        device->base = base;
        device->size = size;
	printf ("Registered flash %s\n", fn->identify(data));
	return 0;
    }
    return -1;
}

void list_flash_devices(int argc, char **argv, void *data)
{
    if (first_flash)
    {
        gbios_flash_device *flash;
        printf("Flash devices:\n");
        for(flash = first_flash; flash; flash = flash->next)
        {
            printf("%s: %s, base=0x%08x, size=0x%x\n",
                    flash->tag, flash->fn->identify(flash->data),
                    flash->base, flash->size);   
        }   
    }
    else
        printf("No flash registered\n"); 
}

void progself_flash_devices(int argc, char **argv, void *data)
{
    if (argc < 1)
    {
        printf("Must specify flash device\n");
        return;
    }
    
    if (first_flash)
    {
        gbios_flash_device *flash;
        for(flash = first_flash; flash; flash = flash->next)
        {
            if (strcmp(argv[0], flash->tag) == 0)
            {
                extern char __start[];
                extern char __end_text[];
                extern char __start_data[];
                extern char __end_data[];
                unsigned length = (__end_text - __start) + (__end_data - __start_data) + 0x100;
                printf("Program length 0x%08x\n", length);
                printf("Erasing: ");
                if (flash->fn->erase(flash->data, 0x100, length) < 0)
                {
                    printf("Failed\n");
                    return;
                }
                else
                    printf("Complete\n");
                printf("Programming: ");
                if (flash->fn->program(flash->data, 0x100, __start, length) < 0)
                    printf("Failed\n");
                else
                    printf("Complete\n");
            }
        }   
    }
    else
        printf("No flash registered\n");    
}

extern u32 flash_elf (const char *elf_base, u32 flash_base, u32 flash_size);

void elf_flash_device(int argc, char **argv, void *data)
{
    if (argc < 1)
    {
        printf("Must specify flash device\n");
        return;
    }
    
    if (first_flash)
    {
        gbios_flash_device *flash;
        for(flash = first_flash; flash; flash = flash->next)
        {
            if (strcmp(argv[0], flash->tag) == 0)
            {
                flash_elf(TftpLoadAddress, 0xfff00000, flash);
            }
        }   
    }
    else
        printf("No flash registered\n");    
}


void do_flash(int argc, char **argv, void *data)
{
    if (argc)
        search_dictionary(flash_dictionary, argc, argv);
    else
        printf("Specify flash subcommand\n");    
}
