/* $Id: pci.h,v 1.5 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#ifndef _GBIOS_PCI_H
#define _GBIOS_PCI_H

#include "iotypes.h"

int pci_config_read_32(int bus, int dev, int func, int reg, u32 *data);
int pci_config_read_16(int bus, int dev, int func, int reg, u16 *data);
int pci_config_read_8(int bus, int dev, int func, int reg, u8 *data);
void pci_config_write_32(int bus, int dev, int func, int reg, u32 data);
void pci_config_write_16(int bus, int dev, int func, int reg, u16 data);
void pci_config_write_8(int bus, int dev, int func, int reg, u8 data);

struct pci_device *pci_find_device(unsigned vendor, unsigned device, unsigned number);

void pci_scan(int argc, char **argv, void *data);

void *pci_to_local(ioaddr addr);
ioaddr local_to_pci(volatile void *addr);

/* The device structure is created for each device, including bridges. This is
 * persistent. */
struct pci_bus;
struct pci_device
{
    u8 device;
    u8 function;
    u8 type;
    ioaddr region[6];		/* These initially hold the region sizes,
				 * and are replaced with base addresses. */
    struct pci_bus *bus;	/* The containing bus */
    struct pci_device *next;
    struct pci_bus *child;	/* bridge devices only */
};

/* And each bus gets a bus structure. */
struct pci_bus
{
    int bus:8;
    int fast_back_to_back:1;
    int have_io_space:1;		/* Not all bridges have this. */
    int have_prefetch_space:1;	/* Not all bridges have this. */
    struct pci_device *device;
    struct pci_bus *parent;
    struct pci_bus *first_child;
    struct pci_bus *next;	/* Next child of the same parent */
    struct pci_device *first_dev;
    struct pci_region *spaces[3];	/* IO, MMIO, Prefetchable */
};

extern struct pci_bus pci_root;

void pci_configure_device (struct pci_bus *bus, int device, int function);
void pci_set_command(struct pci_bus *bus);

#define PCI_REGION_IO 0
#define PCI_REGION_MMIO 1
#define PCI_REGION_PREFETCH 2

unsigned pci_assign_space (struct pci_bus *bus, int regtype, ioaddr limit,
		       ioaddr lower);
void pci_print_bus (int depth, struct pci_bus *bus);

extern int pci_default_latency;
extern int pci_default_cacheline;


#endif
