/* $Id: dictionary.c,v 1.7 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <dictionary.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <flash.h>
#include <pci.h>
#include <net.h>
#include <run.h>

void print_help_text(int argc, char **argv, void *data);
void do_getenv(int argc, char **argv, void *data);
void do_setenv(int argc, char **argv, void *data);
void do_memtest(int argc, char **argv, void *data);
void do_mtloop(int argc, char **argv, void *data);
void do_pci_access(int argc, char **argv, void *data);
void do_timeloop(int argc, char **argv, void *data);
void do_test(int argc, char **argv, void *data);

const gbios_helpdata main_help = {
    "Introduction\n", main_dictionary, help_dictionary
};

const gbios_dictionary main_dictionary[] = {
#ifdef HAVE_flash
    {"flash", "FLASH memory commands", do_flash, NULL},
#endif
#ifdef HAVE_pci
    {"pci", "List PCI bus", pci_scan, NULL},
#endif
#ifdef HAVE_net
    {"net", "Network commands", do_net, NULL},
#endif
    {"test", "Self test commands", do_test, NULL},
    {"help", "Print help text", print_help_text, (void *)&main_help},
#ifdef HAVE_NVRAM
    {"getenv", "Get environment variable", do_getenv, NULL},
    {"setenv", "Set environment variable", do_setenv, NULL},
#endif
    {"boot", "Load code by default method", do_boot, NULL},
    {"go", "Run booted code, or at a specified address", do_run, NULL},
    {NULL}
};



const gbios_helpdata test_helpdata = {
    "Test Commands\n", test_dictionary, NULL
};

const gbios_dictionary help_dictionary[] = {
#ifdef HAVE_flash
    {"flash", "FLASH memory commands", print_help_text, (void *)&flash_helpdata},
#endif
#ifdef HAVE_net
    {"net", "Network Commands", print_help_text, (void *)&net_helpdata},
#endif
    {"test", "Test Commands", print_help_text, (void *)&test_helpdata},
    {NULL}
};

void search_dictionary(const gbios_dictionary *dict, int argc, char **argv)
{
    int i;
    int len;
    int found = -1;
    
    if (argc < 1)
        return;
    
    len = strlen(argv[0]);
    for(i = 0; dict[i].word; i++)
    {
           if (strncmp(argv[0], dict[i].word, len))
               continue;
           if (found != -1)
           {
               printf("\"%s\" is ambiguous. Try \"help\" for help.\n", argv[0]);
               return;
           }
           found = i;
           if (strlen(dict[i].word) == len)
               break;
    }
    
    if (found == -1)
    {
        printf("\"%s\" not recognised. Try \"help\" for help.\n", argv[0]);
        return;
    }
    
    if (dict[found].action)
        dict[found].action(argc - 1, argv + 1, dict[found].data);
}

void print_help_text(int argc, char **argv, void *data)
{
    const gbios_helpdata *help =data;
    
    if (argc && help->next_dict)
        search_dictionary(help->next_dict, argc, argv);
    else
    {
        const gbios_dictionary *dict;
        if (help->text)
            printf(help->text);
        if (help->dict)
            for(dict = help->dict; dict->word; dict++)
                printf("%-10s: %s\n", dict->word, dict->helptext);
    }
}

#ifdef HAVE_NVRAM
void do_getenv(int argc, char **argv, void *data)
{
    extern  int getenv(const char *name, char *buffer, unsigned size);
    
    if (argc == 0)
        nvram_list();
    else if (argc > 1)
        printf("Usage: getenv name\n");
    else
    {
        char buffer[128];
        printf("Getenv %s\n", argv[0]);
        if (getenv(argv[0], buffer, 128) == 0)
            printf("Got %s\n", buffer);
    }
  
}
void do_setenv(int argc, char **argv, void *data)
{
    extern void putenv(const char *name, const char *value);
    nvram_init(4096);
    if (argc < 1 || argc > 2)
        printf("Usage: setenv name [value]\n");
    else if (argc == 2)
    {
        printf("Setenv %s=%s\n", argv[0], argv[1]);
        putenv(argv[0], argv[1]);
    }
    else
        putenv(argv[0], NULL);
}
#endif

unsigned long simple_strtoul(const char *cp,char **endp,unsigned int base);


static unsigned pci_bus ;
static unsigned pci_dev ;
static unsigned pci_func ;
static unsigned pci_addr ;
static unsigned pci_data ;

void do_pci_access_help(void)
{
    printf("syntax:\n") ;
    printf(" tp cr <reg> <bus> <dev> <func> - config read\n") ;
    printf(" tp cw <data> <reg> <bus> <dev> <func> - config write\n") ;
}

void do_pci_access(int argc, char **argv, void *data)
{  
    if (argc > 0)
        switch (argv[0][0])
        {
            case 'c' : 
                switch (argv[0][1])
                {
                    case 'r' :
                        if (argc > 1)
                            pci_addr = simple_strtoul(argv[1], NULL, 16);
                        if (argc > 2)
                            pci_bus = simple_strtoul(argv[2], NULL, 16);
                        if (argc > 3)
                            pci_dev = simple_strtoul(argv[3], NULL, 16);
                        if (argc > 4)
                            pci_func = simple_strtoul(argv[4], NULL, 16);
                        pci_config_read_32( pci_bus, pci_dev, pci_func,
                            pci_addr, &pci_data ) ;
                        printf("read 0x%08X from configuration register 0x%02X of 0x%02X, 0x%02X, 0x%02X\n",
                                pci_data, pci_addr, pci_bus, pci_dev, pci_func) ;
                        break ;
                    case 'w' :
                        if (argc > 1)
                            pci_data = simple_strtoul(argv[1], NULL, 16);
                        if (argc > 2)
                            pci_addr = simple_strtoul(argv[2], NULL, 16);
                        if (argc > 3)
                            pci_bus = simple_strtoul(argv[3], NULL, 16);
                        if (argc > 4)
                            pci_dev = simple_strtoul(argv[4], NULL, 16);
                        if (argc > 5)
                            pci_func = simple_strtoul(argv[5], NULL, 16);
                        pci_config_write_32( pci_bus, pci_dev, pci_func,
                            pci_addr, pci_data ) ;
                        printf("wrote 0x%08X to configuration register 0x%02X of 0x%02X, 0x%02X, 0x%02X\n",
                                pci_data, pci_addr, pci_bus, pci_dev, pci_func) ;
                        break ;
                    default :
                        do_pci_access_help() ;
                        break ;
                }
                break ;
                
            default :
                do_pci_access_help() ;
                break ;
                
        }
    else
    {
        do_pci_access_help() ;
        return ;
    }
}
