/* $Id: malloc.c,v 1.2 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 1999-2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */
/* Allocate memory within shared region - this is a top-down memory allocator,
   as the standard library malloc may be working up from the bottom. */
   
#include <malloc.h>
   
#define ALLOC_SMALL 8
 
#define _TPZ_message_log(x,...) (void)0

typedef struct malloc_header {
    struct malloc_header *next;
    size_t size;
} malloc_header;   

static malloc_header *free_p;
static void *malloc_base;

void malloc_add_pool(void *base, size_t size)
{
    malloc_header *ptr;
    malloc_header *prev = NULL;
    malloc_header *region = base;

    if ((! malloc_base) || base < malloc_base)
        malloc_base = base;
    
    /* Short cut the initial case to avoid trouble. */
    if (! free_p)
    {
        region->size = size;
        region->next = NULL;  
        free_p = region;
        return;
    }
      
    /* Also short cut adding to the top of the pile */
    if (free_p < region)
    {
        
        if (free_p->size + (unsigned) free_p == (unsigned) region)
            free_p->size += size;
        else
        {
            region->next = free_p;
            region->size = size;
            free_p = region;
        }
        return;
    }
      
    for(ptr = free_p; ptr->next > region ; prev=ptr, ptr = ptr->next)
    {
        _TPZ_message_log("Free 0x%08x - check 0x%08x", region, ptr);
        /* if (ptr < (malloc_header *)malloc_base)
            _TPZ_message_fatal("Escaped known region"); */
    }
    /* We now have a ptr -> region just above, and prev->region above that */
            
    /* Coalesce on lower side */
    if (ptr->next &&
            ptr->next->size + (unsigned) ptr->next == (unsigned) region)
    {
        region = ptr->next;
        region->size += size;
    }
    else
    {
        region->next = ptr->next;
        region->size = size;  
    }
    
    /* If immediately below, coalesce. */
    if (region->size + (unsigned) region == (unsigned) ptr)
    {
        region->size += ptr->size;
        if (prev)
            prev->next = region;
        else
            free_p = region;
    }
    
}

void free(void *ptr)
{
    malloc_header *header = ((malloc_header *) ptr) - 1;
    
    malloc_add_pool(header, header->size);
}

void *malloc(size_t size)
{
    malloc_header *ptr;
    malloc_header *prev = NULL;
    
    /* Align size requirement */
    size += 2 * sizeof(malloc_header) - 1;
    size &= ~(sizeof(malloc_header) - 1);
    
    for(ptr = free_p; ptr; prev = ptr, ptr = ptr->next)
    {
        /* Subdivide a region - return the top segment. */
        if (ptr->size > size + ALLOC_SMALL)
        {
            malloc_header *region;
            ptr->size -= size;
            region = (malloc_header *) 
                    ((unsigned)ptr + ptr->size);
            region->size = size;
            return region + 1;
            
        }
        /* Return the whole region. */
        else if (ptr->size >= size)
        {
            if (prev)
                prev->next = ptr->next;
            else
                free_p = ptr->next;
            return ptr + 1;
        }
    }   

    return NULL;
}

#if 0
void malloc_list(void)
{
    malloc_header *ptr;
    
    if (! free_p)
        _TPZ_message_log("No shared memory free");
    else
        for(ptr = free_p; ptr;  ptr = ptr->next)
            _TPZ_message_log("Shared pool: 0x%08x (0x%x)", ptr, ptr->size);
}
#endif
