/* $Id: test.c,v 1.3 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <dictionary.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <flash.h>
#include <bsp.h>
#include <console.h>
#include <io.h>
unsigned long simple_strtoul(const char *cp,char **endp,unsigned int base);

void fill(u32 base, u32 length, u32 data, u32 step)
{
    u32 i;
    
    printf("Fill 0x%08lx to 0x%08lx\n", base, base + length);
    
    for(i = 0; i < length; i += 4)
    {
        *(volatile u32 *)(base + i) = data;
        data += step;
    }   
}


void test(u32 base, u32 length, u32 data, u32 step)
{
    unsigned i;
    unsigned count = 0;
    static unsigned lifetime = 0;
    u32 error_bits = 0;
    printf("Test 0x%08lx to 0x%08lx. ", base, base + length);
    
    for(i = 0; i < length; i += 4)
    {
        u32 x;
        if ((x = *(volatile u32 *)(base + i)) != data)
        {
            if (++count < 16)
                printf("\nError at 0x%08lx: got 0x%08lx expected 0x%08lx",
                    base + i, x, data);
            error_bits |= (x ^ data);
        }
        data += step;
    }  
    lifetime += count;
    if (count)
        printf("\n** Error bits 0x%08lx\n** ", error_bits);
    printf("%d errors (lifetime %d)\n", count, lifetime); 
}


static u32 base = 0;
static u32 length = 0x80000000;
static u32 test_data = 0x31415926;
static u32 step = 0x27182817;

static void do_filltest(u32 base, u32 length,
                 u32 data, u32 step)
{
    u32 top = base + length;
    u32 working_data = data;
    
    /* Check for wraparound */
    if (top > memory_top || top < base)
        top = memory_top;
    if (base < memory_base)
        base = memory_base;
    if (base < bios_base)
    {
        u32 test_top = top > bios_base ? bios_base : top;
        fill(base, test_top - base, data, step);
        working_data = data + ((test_top - base)/4) * step;
    }
    if (top > bios_top)
    {
        u32 test_base = base < bios_top ? bios_top : base;
        fill(test_base, top - test_base, working_data, step);
    }
    if (base < bios_base)
    {
        u32 test_top = top > bios_base ? bios_base : top;
        test(base, test_top - base, data, step);
        data += ((test_top - base)/4) * step;
    }
    if (top > bios_top)
    {
        u32 test_base = base < bios_top ? bios_top : base;
        test(test_base, top - test_base, data, step);
    }
}        

void do_memtest(int argc, char **argv, void *data)
{
    
    if (argc > 0)
        base = simple_strtoul(argv[0], NULL, 16);
    if (argc > 1)
        length = simple_strtoul(argv[1], NULL, 16);
    if (argc > 2)
        test_data = simple_strtoul(argv[2], NULL, 16);
    if (argc > 3)
        step = simple_strtoul(argv[3], NULL, 16);
    if (length > memory_top - memory_base)
    printf("Test from base 0x%08lx length 0x%lx data 0x%08lx step 0x%lx\n",
            base, length, test_data, step);
    do_filltest(base, length, test_data, step);
}

static __inline__ void set_l2cr(unsigned x)
{
    __asm__ volatile("sync; mtspr 1017,%0; sync"::"r"(x));
}
static __inline__ unsigned get_l2cr(void)
{
    unsigned value;
    __asm__ volatile("sync; mfspr %0, 1017":"=r"(value));

    return value;
}


#define L2E     0x80000000
#define L2PE    0x40000000
#define L2SIZ1M 0x30000000
#define L2CLK2  0x08000000
#define L2RAMPB 0x01000000
#define L2DO    0x00400000
#define L2I     0x00200000
#define L2CTL   0x00100000
#define L2WT    0x00080000
#define L2TS    0x00040000
#define L2OH1   0x00010000
#define L2SL    0x00001000
#define L2IP    0x00000001

void l2_cache_set(unsigned cachectl)
{

    /* Disable dangerous bits from SERPROM */    
    cachectl &= ~(L2E | L2TS);
    
             
        printf("Setting L2: 0x%08x\n", cachectl);
        set_l2cr(cachectl);
        set_l2cr(cachectl | L2I);

        while (get_l2cr() & L2IP)
	    ;

        set_l2cr(cachectl | L2E);
   
}

void do_mtloop(int argc, char **argv, void *data)
{  
    int i = 0;
    u32 working_data;
    if (argc > 0)
        base = simple_strtoul(argv[0], NULL, 16);
    if (argc > 1)
        length = simple_strtoul(argv[1], NULL, 16);
    if (argc > 2)
        test_data = simple_strtoul(argv[2], NULL, 16);
    if (argc > 3)
        step = simple_strtoul(argv[3], NULL, 16);
    working_data = test_data;
    l2_cache_set(0x39890000);
    for(;;)
    {
    printf("%d: Test from base 0x%08lx length 0x%lx data 0x%08lx step 0x%lx\n",
            i, base, length, working_data, step);
    do_filltest(base, length, working_data, step);
    working_data += (1 + length/4) * step;
    if (console_input() != -1)
        return;
    i++;
    }
}

static unsigned long target_addr;

void do_r8(int argc, char **argv, void *data)
{  
    if (argc > 0)
        target_addr = simple_strtoul(argv[0], NULL, 16);
    printf("0x%08lx: 0x%02x\n", target_addr,
             *(volatile u8 *)target_addr);
}

void do_w8(int argc, char **argv, void *data)
{  
    u8 working_data = 0;
    if (argc > 1)
    {
        target_addr = simple_strtoul(argv[0], NULL, 16);
        working_data = simple_strtoul(argv[1], NULL, 16);
    }
    else if (argc == 1)
        working_data = simple_strtoul(argv[0], NULL, 16);
    else if (argc == 0)
    {
        printf("test w8 [addr] data\n");
        return;
    }
    *(volatile u8 *)target_addr = working_data;
}


void do_r16(int argc, char **argv, void *data)
{  
    if (argc > 0)
        target_addr = simple_strtoul(argv[0], NULL, 16);
    printf("0x%08lx: 0x%04x\n", target_addr,
             *(volatile u16 *)target_addr);
}

void do_w16(int argc, char **argv, void *data)
{  
    u16 working_data = 0;
    if (argc > 1)
    {
        target_addr = simple_strtoul(argv[0], NULL, 16);
        working_data = simple_strtoul(argv[1], NULL, 16);
    }
    else if (argc == 1)
        working_data = simple_strtoul(argv[0], NULL, 16);
    else if (argc == 0)
    {
        printf("test w16 [addr] data\n");
        return;
    }
    *(volatile u16 *)target_addr = working_data;
}


void do_r32(int argc, char **argv, void *data)
{  
    if (argc > 0)
        target_addr = simple_strtoul(argv[0], NULL, 16);
    printf("0x%08lx: 0x%08lx\n", target_addr,
             *(volatile u32 *)target_addr);
}

void do_w32(int argc, char **argv, void *data)
{  
    u32 working_data = 0;
    if (argc > 1)
    {
        target_addr = simple_strtoul(argv[0], NULL, 16);
        working_data = simple_strtoul(argv[1], NULL, 16);
    }
    else if (argc == 1)
        working_data = simple_strtoul(argv[0], NULL, 16);
    else if (argc == 0)
    {
        printf("test w32 [addr] data\n");
        return;
    }
    *(volatile u32 *)target_addr = working_data;
}

#ifdef FIXUP_TPV4
void gmv1_program (int argc, char **argv);
#endif
#ifdef HOST_BRIDGE_107
void dump_107 (int argc, char **argv);
#endif

const gbios_dictionary test_dictionary[] = {
    {"mem", "Memory test", do_memtest, NULL},
    {"loop", "Memory loop test", do_mtloop, NULL},
    {"r8", "Read byte", do_r8, NULL},
    {"r16", "Read short", do_r16, NULL},
    {"r32", "Read long", do_r32, NULL},
    {"w8", "Write byte", do_w8, NULL},
    {"w16", "Write short", do_w16, NULL},
    {"w32", "Write long", do_w32, NULL},
#ifdef FIXUP_TPV4
    {"gmv1rom", "Write ROM on GMV1", gmv1_program, NULL},
#endif
#ifdef HOST_BRIDGE_107
    {"107", "Dump MPC107 registers", dump_107, NULL},
#endif
    {NULL}
};

void do_test(int argc, char **argv, void *data)
{
    if (argc)
        search_dictionary(test_dictionary, argc, argv);
    else
        printf("Specify test subcommand\n");    
}
