/*
 *	LiMon - BOOTP.
 *
 *	Copyright 1994, 1995, 2000 Neil Russell.
 *	(See License)
 */

#include	<net.h>
#include <stdio.h>
#include	"bootp.h"
#include <timer.h>

#define DEBUG
#define TIMEOUT		15		/* Seconds before trying BOOTP again */

#define PORT_BOOTPS	67		/* BOOTP server UDP port */
#define PORT_BOOTPC	68		/* BOOTP client UDP port */


u32_t		BootpID;
char		BootFile[128];
int		BootpTry;


/*
 *	Handle a BOOTP received packet.
 */
static void
BootpHandler(u8_t * pkt, unsigned dest, unsigned src, unsigned len)
{
	Bootp_t *	bp;


#if 0
	printf("got BOOTP packet (src=%d, dst=%d, len=%d)\n", src, dest, len);
#endif /* DEBUG */

	bp = (Bootp_t *)pkt;

	if (dest != PORT_BOOTPC || src != PORT_BOOTPS)
		return;
	if (len != sizeof (Bootp_t))
		return;
	if (bp->bp_op != OP_BOOTREPLY)
		return;
	if (bp->bp_htype != HWT_ETHER)
		return;
	if (bp->bp_hlen != HWL_ETHER)
		return;
	if (bp->bp_id != BootpID)
		return;

	/*
	 *	Got a good BOOTP reply.  Copy the data into our variables.
	 */
	NetOurIP = bp->bp_yiaddr;
	NetServerIP = bp->bp_siaddr;
	NetCopyEther(NetServerEther, ((Ethernet_t *)NetRxPkt)->et_src);
	memcpy(BootFile, bp->bp_file, sizeof bp->bp_file);
	NetSetTimeout(0, (thand_f *)0);

#ifdef DEBUG
	printf("Got good BOOTP\n");
        printf("Our IP: ");
        NetPrintIPaddr(NetOurIP);
        printf("\nServer IP: ");
        NetPrintIPaddr(NetServerIP);
        printf("\nBootfile: %s\n", BootFile);
#endif /* DEBUG */

        NetState = NETLOOP_SUCCESS;
	/* TftpStart(); */
}


/*
 *	Timeout on BOOTP request.  Try again, forever.
 */
static void
BootpTimeout(void)
{
	BootpRequest();
}


void
BootpRequest(void)
{
	volatile u8_t *	pkt;
	Bootp_t *	bp;


	printf("BOOTP broadcast %d\r", ++BootpTry);

	pkt = NetTxPacket;

	NetSetEther(pkt, NetBcastAddr, PROT_IP);
	pkt += ETHER_HDR_SIZE;

	NetSetIP(pkt, 0xffffffffL, PORT_BOOTPS, PORT_BOOTPC, sizeof (Bootp_t));
	pkt += IP_HDR_SIZE;

	bp = (Bootp_t *)pkt;
	bp->bp_op = OP_BOOTREQUEST;
	bp->bp_htype = HWT_ETHER;
	bp->bp_hlen = HWL_ETHER;
	bp->bp_hops = 0;
	bp->bp_secs = htons(ticks_since_boot() / get_hz());
	bp->bp_ciaddr = 0;
	bp->bp_yiaddr = 0;
	bp->bp_siaddr = 0;
	bp->bp_giaddr = 0;
	NetCopyEther(bp->bp_chaddr, NetOurEther);

	/*
	 *	Bootp ID is the lower 4 bytes of our ethernet address
	 *	plus the current time in HZ.
	 */
	BootpID = ((u32_t)NetOurEther[2] << 24)
		| ((u32_t)NetOurEther[3] << 16)
		| ((u32_t)NetOurEther[4] << 8)
		| (u32_t)NetOurEther[5];
	BootpID += ticks_since_boot();
	bp->bp_id = BootpID;

	NetSendPacket(NetTxPacket, BOOTP_SIZE);

	NetSetTimeout(TIMEOUT * get_hz(), BootpTimeout);
	NetSetHandler(BootpHandler);
}
