/*
 *	LiMon - BOOTP.
 *
 *	Copyright 1994, 1995, 2000 Neil Russell.
 *	(See License)
 */

#ifndef __BOOTP_H__
#define __BOOTP_H__

#ifndef __NET_H__
#include	"net.h"
#endif /* __NET_H__ */

/**********************************************************************/

/*
 *	BOOTP header.
 */
typedef struct
{
	u8_t		bp_op;		/* Operation */
#   define OP_BOOTREQUEST   1
#   define OP_BOOTREPLY	    2
	u8_t		bp_htype;	/* Hardware type */
#   define HWT_ETHER	    1
	u8_t		bp_hlen;	/* Hardware address length */
#   define HWL_ETHER	    6
	u8_t		bp_hops;	/* Hop count (gateway thing) */
	u32_t		bp_id;		/* Transaction ID */
	u16_t		bp_secs;	/* Seconds since boot */
	u16_t		bp_spare1;	/* Alignment */
	IPaddr_t	bp_ciaddr;	/* Client IP address */
	IPaddr_t	bp_yiaddr;	/* Your (client) IP address */
	IPaddr_t	bp_siaddr;	/* Server IP address */
	IPaddr_t	bp_giaddr;	/* Gateway IP address */
	u8_t		bp_chaddr[16];	/* Client hardware address */
	char		bp_sname[64];	/* Server host name */
	char		bp_file[128];	/* Boot file name */
	char		bp_vend[64];	/* Vendor information */
}
		Bootp_t;

#define BOOTP_HDR_SIZE	sizeof (Bootp_t)
#define BOOTP_SIZE	(ETHER_HDR_SIZE + IP_HDR_SIZE + BOOTP_HDR_SIZE)

/**********************************************************************/
/*
 *	Global functions and variables bootp.c */
extern u32_t	BootpID;		/* ID of cur BOOTP request */
extern char	BootFile[128];		/* Boot file name */

extern void	BootpRequest(void);	/* Send a BOOTP request */

/**********************************************************************/

#endif /* __BOOTP_H__ */
