/* $Id: belfry.c,v 1.4 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <ppc.h>

/* Lower BAT
 * 25 - W  - Write through mode
 * 26 - I  - Cache inhibited
 * 27 - M  - Memory coherence
 * 28 - G  - Guarded 
 */
#define BAT_W  0x40
#define BAT_I  0x20
#define BAT_M  0x10
#define BAT_G  0x08
#define BAT_RW 0x02

/* Upper BAT
 * 30 - VS - Supervisor mode valid
 * 31 - VP - User mode valid
 */
#define BAT_VS 0x02
#define BAT_VP 0x01

static void __setibat0 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 528, %0; mtspr 529, %1; isync"::"r" (u), "r" (l));
}
static void __setdbat0 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 536, %0; mtspr 537, %1; isync"::"r" (u), "r" (l));
}
static void __setibat1 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 530, %0; mtspr 531, %1; isync"::"r" (u), "r" (l));
}
static void __setdbat1 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 538, %0; mtspr 539, %1; isync"::"r" (u), "r" (l));
}
static void __setibat2 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 532, %0; mtspr 533, %1; isync"::"r" (u), "r" (l));
}
static void __setdbat2 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 540, %0; mtspr 541, %1; isync"::"r" (u), "r" (l));
}
static void __setibat3 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 534, %0; mtspr 535, %1; isync"::"r" (u), "r" (l));
}
static void __setdbat3 (unsigned u, unsigned l)
{
    __asm__ ("mtspr 542, %0; mtspr 543, %1; isync"::"r" (u), "r" (l));
}

static unsigned __make_ubat (unsigned address, unsigned length)
{
    unsigned target = 128 * 1024;	/* Smallest BAT is 128 kB */
    unsigned bits = 2;		/* Start the bit mask to shift up */

    do
    {
	if (length <= target)
	    break;
	bits = (bits << 1) | 2;
	target *= 2;
    }
    while (target <= 256 * 1024 * 1024);

    /*
     * This could be a serious error - a BAT must be aligned on a multiple
     * of its size
     */
    if (address % target != 0)
	return 0;

    return address | bits | BAT_VS | BAT_VP;
}

static unsigned __make_lbat (unsigned address, unsigned mode)
{
    /* Address not properly aligned */
    if (address & 0x0001ffff)
	return 0;

    return address | mode | 2;
}

void ppc_set_ibats (unsigned base, unsigned length)
{
    __setibat0 (__make_ubat (base, length), __make_lbat (base, 0));
    __setibat1 (0, 0);
    __setibat2 (0, 0);
    __setibat3 (0, 0);
}
void ppc_set_ibats_reloc(unsigned virtual, unsigned physical, unsigned length)
{
    __setibat0 (__make_ubat (virtual, length), __make_lbat (physical, 0));
    __setibat1 (__make_ubat (physical, length), __make_lbat (physical, 0));
    __setibat2 (0, 0);
    __setibat3 (0, 0);
}
void ppc_set_ibats_reloc2(unsigned virtual1, unsigned virtual2,
         unsigned physical, unsigned length)
{
    __setibat0 (__make_ubat (virtual1, length), __make_lbat (physical, 0));
    __setibat1 (__make_ubat (virtual2, length), __make_lbat (physical, 0));
    __setibat2 (__make_ubat (physical, length), __make_lbat (physical, 0));
    __setibat3 (0, 0);
}

void ppc_set_memory_dbat (unsigned length)
{
    __setdbat0 (__make_ubat (0, length), __make_lbat (0, BAT_M));
}

void ppc_set_io_dbat (int bat, unsigned base, unsigned length)
{
    unsigned ubat = __make_ubat (base, length);
    unsigned lbat = __make_lbat (base, BAT_I | BAT_G);

    switch (bat)
    {
    case 1:
	__setdbat1 (ubat, lbat);
	break;
    case 2:
	__setdbat2 (ubat, lbat);
	break;
    case 3:
	__setdbat3 (ubat, lbat);
	break;
    }
}

void ppc_set_mem_dbat_reloc (int bat, unsigned base, unsigned phys, unsigned length)
{
    unsigned ubat = __make_ubat (base, length);
    unsigned lbat = __make_lbat (phys, BAT_M);

    switch (bat)
    {
    case 1:
	__setdbat1 (ubat, lbat);
	break;
    case 2:
	__setdbat2 (ubat, lbat);
	break;
    case 3:
	__setdbat3 (ubat, lbat);
	break;
    }
}

void ppc_set_io_dbat_reloc (int bat, unsigned base, unsigned phys, unsigned length)
{
    unsigned ubat = __make_ubat (base, length);
    unsigned lbat = __make_lbat (phys, BAT_I | BAT_G);

    switch (bat)
    {
    case 1:
	__setdbat1 (ubat, lbat);
	break;
    case 2:
	__setdbat2 (ubat, lbat);
	break;
    case 3:
	__setdbat3 (ubat, lbat);
	break;
    }
}

void ppc_set_rom_dbat (int bat, unsigned base, unsigned length)
{
    unsigned ubat = __make_ubat (base, length);
    unsigned lbat = __make_lbat (base, BAT_W);

    switch (bat)
    {
    case 1:
	__setdbat1 (ubat, lbat);
	break;
    case 2:
	__setdbat2 (ubat, lbat);
	break;
    case 3:
	__setdbat3 (ubat, lbat);
	break;
    }
}
