/* $Id: main.c,v 1.6 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <console.h>
#include <malloc.h>
#include <bsp.h>
#include <version.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <dictionary.h>
#include <board.h>

#define MAX_TOKENS 8
static char *tokens[MAX_TOKENS+1];

static char *tokenize_word(char *cmdline, char **remainder)
{
    char *word = cmdline;
    
    for(; *word && isspace(*word); word++)
        ;
    cmdline = word;
    if (! *word)
        return NULL;
    
    for(; *cmdline && ! isspace(*cmdline); cmdline++)
        ;
    if (*cmdline)
        *remainder = cmdline+1;
    else
        *remainder = NULL;
    *cmdline = 0;
    return word;
}

static int tokenize(char **tokens, char *cmdline)
{
    int i;
    for(i = 0; i < MAX_TOKENS; i++)
    {
        char *remainder;
        tokens[i] = tokenize_word(cmdline, &remainder);
        if (! tokens[i])
            break;
        if (! remainder)
        {
            tokens[++i] = NULL;
            break;
        }
        cmdline = remainder;
    }
    return i;
}


static void output_string(const char *x)
{
    int i;
    for(i = 0; *x; x++, i++)
        console_outchar(*x);
    console_outchar(' ');
    for(; i > 0; i--)
        console_outchar('\b');   
}

int main(void)
{
    bsp_init_post_reloc();
         
    printf("GBIOS release %s (compiled %s)\n", GBIOS_VERSION, __DATE__);
    
    bsp_init_post_hello();
    
    bsp_identify();
    
    for(;;)
    {
        char buffer[70];
        int x = -1;     
        int i = 0;
        int last = 0;
        int escape = 0;
        int escape_bracket = 0;
        int escape_bracket_3 = 0;
        
        buffer[0] = 0;
        console_outchar('>');
        while(x != '\r')
        {    
            /* Check mailboxes */
            bsp_mailbox_check();            
            
            x = console_input();   
            if (x != -1)
            {
                /* Cursor keys register as "ESC[A" to "ESC[D" */
                if (escape)
                {
                    escape = 0;
                    if (x == '[')
                    {
                        escape_bracket = 1;
                        continue;
                    }
                    else
                        printf("Esc\n");
                }
                if (escape_bracket)
                {
                    escape_bracket = 0;
                    if (x == 'D')
                    {
                        if (i > 0)
                        {
                            i--;
                            console_outchar('\b');
                        }                      
                        continue;
                    }
                    else if (x == 'C')
                    {
                        if (i < last)
                        {
                            console_outchar(buffer[i]);
                            i++;
                        }
                        continue;
                    }
                    if (x == 'A' || x == 'B')
                        continue;
                    if (x == '3')
                    {
                        escape_bracket_3 = 1;
                        continue;
                    }
                    else
                        printf("Esc[\n");
                }
                if (escape_bracket_3)
                {
                    escape_bracket_3 = 0;
                    if (x == '~')
                        x = 127;
                }
                if (isprint(x) && last < 255)
                {
                    int j;
                    for(j = last+1; j > i; j--)
                        buffer[j] = buffer[j-1];
                    buffer[i] = x;
                    output_string(buffer + i);
                    i++;
                    last++;
                }
                else switch(x) {
                    case 8: /* backspace */
                        if (i > 0)
                        {
                            int j;
                            console_outchar('\b');
                            output_string(buffer + i);
                            console_outchar('\b');
                            for(j = i; j < last+1; j++)
                                buffer[j] = buffer[j+1];
                            i--;
                            last--;
                            buffer[last] = 0;
                        }
                        break;
                    case 127: /* del */
                        if (i < last)
                        {
                            int j;
                            for(j = i; j < last+1; j++)
                                buffer[j] = buffer[j+1];
                            output_string(buffer + i);
                            console_outchar('\b');
                            last--;
                            buffer[last] = 0;
                        }
                        break;
                    case 13: /* enter */
                        console_outchar('\n');
                        break;
                    case 27: /* escape */
                        escape=1;
                        break;
                    case 10:
                        break;
                    default:
                        printf("\nGot %d\n", x);
                }
            }
        }
        search_dictionary(main_dictionary, tokenize(tokens, buffer), tokens);
    }
    
    bsp_indicate_dead();   
}

RODATA const void (*init2_ptr)(void) = main;

