/* $Id: setup.c,v 1.6 2000/11/15 17:44:55 apc Exp $ */
/* Copyright 2000  AG Electronics Ltd. */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include "ppcreg.h"
#include <ppc.h>
#include <stdio.h>
#include <asmstart.h>

unsigned __getmsr(void)
{
    unsigned result;   
    __asm__ volatile ("mfmsr %0" : "=r" (result));
    return result;
}
unsigned __gethid0(void)
{
    unsigned result;
    __asm__ volatile ("mfspr %0,1008" : "=r" (result));
    return result;
}
unsigned __gethid1(void)
{
    unsigned result;
    __asm__ volatile ("mfspr %0,1009" : "=r" (result));
    return result;
}
void __sethid0(unsigned value)
{
    __asm__ volatile ("mtspr 1008,%0" : : "r" (value));
}

int __getpvr(void)
{
    int result;
    __asm__("mfspr %0, 287" : "=r" (result));
    return result;
}

void __setmsr(unsigned value)
{
    __asm__ volatile ("mtmsr %0; sync" :: "r" (value));   
}

void __set1015(unsigned value)
{
    __asm__ volatile ("mtspr 1015,%0" : : "r" (value));
}

extern void _init_float_registers(double *);
static double dummy_float = 1.0;
#ifdef BSP_USE_FAKERAM
#define HID0_DCACHE HID0_DCE
#define MSR_DATA MSR_DR
#else
#define HID0_DCACHE 0
#define MSR_DATA 0
#endif

void ppc_setup_cpu(int icache)
{
    int type = __getpvr() >> 16;
    int version = __getpvr() & 0xffff;
    
    if (type == 0xc) 
    {
        if (version == 0x0200)
            __set1015(0x19000004);
        else if (((version & 0xff00) == 0x0200) && 
            (version != 0x0209))
            __set1015(0x01000000);
    }
    if (icache)
    {
        __sethid0(HID0_NHR | HID0_ENBHT | HID0_ICE | HID0_ICFI | HID0_BTIC
                | HID0_DCACHE);
        __sethid0(HID0_DPM | HID0_NHR | HID0_ENBHT | HID0_ICE | HID0_BTIC
                | HID0_DCACHE);        
    }
    else
    {
        __sethid0(HID0_DPM | HID0_NHR | HID0_ENBHT | HID0_BTIC | HID0_DCACHE);
    }
#if 1
    if (type == 8 || type == 12)
    {
        __setmsr(MSR_FP | MSR_DATA);
        _init_float_registers(&dummy_float);
    }
#endif
}

void ppc_identify(void)
{
    int type = __getpvr() >> 16;
    int version = __getpvr() & 0xffff;
    const char *cpu_string = 0;
    switch(type) {
        case 1:
            cpu_string = "601";
            break;
        case 3:
            cpu_string = "603";
            break;
        case 4:
            cpu_string = "604";
            break;
        case 6:
            cpu_string = "603e";
            break;
        case 7:
            cpu_string = "603ev";
            break;
        case 8:
            cpu_string = "750";
            break;
        case 9:
            cpu_string = "604e";
            break;
        case 10:
            cpu_string = "604ev5 (MachV)";
            break;
        case 12:
            cpu_string = "7400";
            break;
        case 50:
            cpu_string = "821";
            break;
        case 80:
            cpu_string = "860";
            break;
    }
    if (cpu_string)
        printf("PowerPC %s", cpu_string);       
    else
        printf("PowerPC unknown (%d)", type);
    printf(" CPU, version %d.%d\n", version >> 8, version & 0xff);       
}

void ppc_enable_dcache(void)
{
#ifndef BSP_USE_FAKERAM
    unsigned hid0 = __gethid0();
    __sethid0(hid0 | HID0_DCI | HID0_DCE);
    __sethid0(hid0 | HID0_DCE);
#endif
}

void ppc_disable_dcache(void)
{
    unsigned hid0 = __gethid0();
    __sethid0(hid0 & ~HID0_DCE);
}

void ppc_enable_mmu(void)
{
    unsigned msr = __getmsr();
    __setmsr(msr | MSR_DR | MSR_IR); 
}

void make_coherent(void *base, unsigned length)
{
    unsigned i;
    
    unsigned hid0 = __gethid0();
    
    if (hid0 & HID0_DCE)
    {
        for(i = 0; i < length; i+= 32)
            __asm__ volatile ("dcbf %1,%0" : : "r" (base), "r" (i));
        if (hid0 & HID0_ICE)
            for(i = 0; i < length; i+= 32)
                __asm__ volatile ("icbi %1,%0" : : "r" (base), "r" (i));
    }
}
